/*============================================================================
  File:     IndexingForJoins.sql

  Summary:  Using just the tools - how can you improve this query?!
  
  Date:     March 2012

  SQL Server Version: 2005/2008/R2/2012
------------------------------------------------------------------------------
  Written by SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE CREDIT
go

-- These samples use the Credit database. You can download and restore the
-- credit database from here:
-- http://www.sqlskills.com/PastConferences.asp

-- Let's start to analyze some queries and their performance:
SET STATISTICS IO ON
go

SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c
		INNER JOIN dbo.provider AS p 
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s 
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r 
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go

-------------------------------------------------------------------------------
-- First, let's get this plan as our base test case... review the 
-- showplan and FORCE every single index listed.
-------------------------------------------------------------------------------

-- USE:
--	0 = Table Scan
--	1 = Clustered Index Seek/Scan
--	name = for all non-clustered indexes 
--		(note/fyi - 2008 NC IDs are 2-250 and 256-1005)
--		(or name for all, name is a bit safer)

SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (1))
		INNER JOIN dbo.provider AS p WITH (INDEX (1))
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m WITH (INDEX (1)) 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s WITH (INDEX (statement_ident))
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r WITH (INDEX (region_ident))
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go

-- Review the Missing Index DMVs recommendation index first...
-- Next, run it through DTA - are they different?

-- Baseline
SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (1))
		INNER JOIN dbo.provider AS p WITH (INDEX (1))
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m WITH (INDEX (1)) 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s WITH (INDEX (1))
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r WITH (INDEX (region_ident))
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go

-- Missing Index DMVs recommendation (modify query for hint)
SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (DMVsRec))
		INNER JOIN dbo.provider AS p 
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s 
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r 
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go

-- DTA's recommendation (modify query for hint)
SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c WITH (INDEX ([DTA_charge__K2_K6_K3_K7]))
		INNER JOIN dbo.provider AS p 
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s 
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r 
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go

-- No hints - what does SQL Server do??
SELECT c.statement_no
		, s.statement_dt
		, c.charge_amt
		, p.provider_name
		, m.lastname
	FROM dbo.Charge AS c 
		INNER JOIN dbo.provider AS p 
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s 
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r 
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
OPTION (MAXDOP 1)
go
